@@ -8,7 +8,7 @@ use CGI();
 BEGIN {
     use Exporter ();
     use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION     = '0.19';
+    $VERSION     = '0.20';
     @ISA         = qw(Exporter);
     #Give a hoot don't pollute, do not export more than needed by default
     @EXPORT      = qw();
@@ -183,6 +183,12 @@ CGI::OptimalQuery - dynamic SQL query viewer
     }
   );
 
+  # get the count
+  $oq->get_count();
+
+  # get link back to this report
+  $oq->get_link();
+
   # output view to STDOUT
   $oq->output();
 
@@ -238,6 +244,10 @@ provides OptimalQuery a connection to the database.
 
 Default fields to show user when loading OptimalQuery the first time. This can also be set as a CGI param where the value is a comma separated list of column aliases. Example: "[NAME], [DOB]".
 
+=item B<< params => { show => "COLALIAS1,COLALIAS2", filter => "[COLALIAS1]=5", sort => "[COLALIAS1]", queryDescr => "descr here", hiddenFilter => "[COLALIAS1]=5" }
+
+Get user set parameters from this hash instead of CGI params. Useful if you are constructing an OQ that should not be influenced by CGI params.
+
 =item B<< select => { SELECT_ALIAS => [ DEP, SQL, NAME, OPTIONS ], .. } >>
 
 The select configuration describes what fields from the query can be selected, filtered, and sorted. 
@@ -266,6 +276,21 @@ The following KEY/VALUES below describe OPTIONS used by the select configuration
 
 =over
 
+=item B<< select => [COLALIAS1, COLALIAS2]
+=item B<< select => "COLALIAS1, COLALIAS2"
+
+Define other select fields to be included in executed SQL. These fields can be used in custom formatters including the built in CGI::OptimalQuery::Base::recview_formatter.
+
+=item B<< formatter => sub { my ($val, $rec, $o, $colAlias) = @_; return $val; } >>
+=item B<< html_formatter => sub { my ($val, $rec, $o, $colAlias) = @_; return $val; } >>
+
+Override the default function that is used to output a field value.
+
+=item B<< formatter => \&CGI::OptimalQuery::Base::recview_formatter
+=item B<< html_formatter => \&CGI::OptimalQuery::Base::recview_html_formatter
+
+Built in formatters to display all field/values specified in 'select'.
+
 =item B<< is_hidden => 1 >>
 
 hides the select field and data from being viewed by the user. Data for this select is still available in callbacks and can be included in the hiddenFilter.
@@ -647,6 +672,7 @@ Saved Searches are stored in a table in the database pointed to by the database
     alert_last_dt DATETIME,
     alert_err TEXT,
     alert_uids LONGTEXT,
+    is_default TINYINT(1) NOT NULL DEFAULT 0,
     CONSTRAINT unq_oq_saved_search UNIQUE (user_id,uri,oq_title,user_title)
   );
 
@@ -667,6 +693,7 @@ Saved Searches are stored in a table in the database pointed to by the database
     alert_last_dt DATE,
     alert_err CLOB,
     alert_uids CLOB,
+    is_default NUMBER DEFAULT 0 NOT NULL,
     CONSTRAINT unq_oq_saved_search UNIQUE (user_id,uri,oq_title,user_title)
   );
   CREATE SEQUENCE s_oq_saved_search;
