@@ -211,7 +211,7 @@ if (! window._OQAjaxQueryLoaded)
           $tr.remove();
         }
         else
-          alert('Could not delete report.');
+          alert('Could not delete report.' + jqXHR.responseText);
       }
     });
     return false;
@@ -277,6 +277,12 @@ if (! window._OQAjaxQueryLoaded)
         return true;
       }
     }
+    
+    // set this saved search as default
+    if($('#OQsave_search_default').length > 0) {
+      dat.save_search_default = $('#OQsave_search_default').prop('checked') ? 1 : 0;
+    }
+    
     $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'json',
       error: function(){
         OQnotify("Could not save report.");
